@@ -26,23 +26,29 @@ namespace FSpot.Filters {
 		{
 		}
 			
-		public bool Convert (string source, string dest)
+		public bool Convert (FilterRequest req)
 		{
 			// FIXME this should copy metadata from the original
 			// even when the source is not a jpeg
-			ImageFile img = ImageFile.Create (source);
-			if (img is JpegFile)
-				return false;
+			string source = req.Current.LocalPath;
 
-			Exif.ExifData exif_data;
-			try {
-				exif_data = new Exif.ExifData (source);
-			} catch (Exception) {
-				exif_data = new Exif.ExifData();
+			using (ImageFile img = ImageFile.Create (req.Current)) {
+				if (img is JpegFile)
+					return false;
+
+				req.Current = req.TempUri ("jpg");
+				string dest = req.Current.LocalPath;
+
+				Exif.ExifData exif_data;
+				try {
+					exif_data = new Exif.ExifData (source);
+				} catch (Exception) {
+					exif_data = new Exif.ExifData();
+				}
+
+				PixbufUtils.SaveJpeg (img.Load(), dest, (int) quality, exif_data);
 			}
 
-			PixbufUtils.SaveJpeg (img.Load(), dest, (int) quality, exif_data);
-
 			return true;
 		}
 	}
